static double
domain_distance(const Domain &domain)
{
    PPL::Constraint_System cs;
    int i;
    for (int i = 0; i < OBJECTIVE_COUNT; i++) {
        const struct variable_objective *o = &variable_objectives[i];
        PPL::Variable x(o->index);
        cs.insert(x * FOLTL_PRECISION == o->value * FOLTL_PRECISION);
    }
    PPL::NNC_Polyhedron objective(cs);
    if (PPL::check_containment(objective, domain)) {
        return 0;
    }
    double result = INFINITY;
    for (auto &polyhedron: domain) {
        double distance = 0;
        for (auto &generator: polyhedron.pointset().generators()) {
            for (auto &objective_generator: objective.generators()) {
                PPL::Checked_Number<mpq_class, PPL::Extended_Number_Policy> e_d;
                if (PPL::euclidean_distance_assign(
                        e_d, generator, objective_generator,PPL::ROUND_UP)) {
                    distance += e_d.raw_value().get_d();
                }
            }
        }
        if (distance < result) {
            result = distance;
        }
    }
    return result;
}
